

/*
   Все, проект рабочий.
*/

#include <TimerOne.h>                                       // Да! Будем работать с прерываниями по таймеру
#include <LiquidCrystal.h>
// #include <FileIO.h>
#include <Console.h>
#include <stdint.h>

#define ENCODER_INCREASE 2

int numberOfStations = 0;                                  // Количество станций в плейлисте

static volatile int EncData = 0;                            // Три переменные для энкодера
static volatile int OldEncData = 0;                         // Кстати, рукоятку вешаем на А4 и А5, а кнопку - на А3
static uint8_t EncState;
byte ChangeMode;                                            // Для проверки действий с энкодером за пределами прерываний

int nowPlaying = 0;                                         // Нужно будет для выбора станций

unsigned long TimeToReturn;                                 // Время для ускорения задумчивых (выхода из режима выбора станции если ничего не выбрали)


LiquidCrystal lcd(8, 9, 10, 11, 12, 13);                    // Куда цепляем LCD

String info = " Jiraffe Radio";                             // Это для затравки
String oldInfo = "";                                        // Это чтобы не перепечатывать лишний раз то что напечатано
String oldMusicInfo = "";                                   // И это тоже
const String clearingRow = "                ";              // Это для чистки строк потом
char to_lcd[] = "                ";                         // Это для печати длинных строк будем использовать
bool RadioPlaying = true;


enum steps {                                                // У программы два шага
    set_station,                                            // Установка станции
    print_info                                              // Выдача информации о том что играет
} stepsOfProgramm;


void setup() {
    Bridge.begin();                                         // Так положено
//    Console.begin();                                        // А это не обязательно, только для отладки

    Timer1.initialize(100);                                 // Устанавливаем таймер на 100 микросекунд
    lcd.begin(16, 2);                                       // Печатаем кое-что важное
    lcd.clear();
    lcd.print(info);
    delay(5000);                                            // Смотрим на это пять секунд и тащимся
    numberOfStations = FGetNumber(1);
    stepsOfProgramm = print_info;                           // Обозначаем следующий шаг программы
    ChangeMode = (PINF & 0b00010011);                       // Читаем и запоминаем остояние трех бит порта  F - то есть ноги А0, А1, А5.

}                                                           // нужно для того, чтобы при проверке и сравнении менять шаги программы


void loop() {
    switch(stepsOfProgramm) {
    case set_station:                                       // Шаг программы - установка станции. Сначала мы его пропускаем.
        nowPlaying = FGetNumber(0);                          // Узнаем, что нынче играет
        FPrintStationNumberAndName(nowPlaying);             // Выдаем на LCD, но уже в другом формате
        TimeToReturn = millis() + 7000;                     // Засекаем время для лечения нерасторопности
        Timer1.attachInterrupt(EncoderScan);                // Начали сканировать энкодер нипадецки, а по прерыванию от таймера
        OldEncData = EncData;                               // и установили начальное значение

        while (stepsOfProgramm != print_info) {             // Будем крутиться до тех пор, пока не утановим другой шаг программы
            int valll = EncData - OldEncData ;              // Сравним старое и новое значение энкодера и запишем в переменную
            OldEncData = EncData;                           // Новое сделаем старым

            if(valll) {                                     // Если переменная сравнения не ноль
                if (valll > ENCODER_INCREASE) nowPlaying++; // меняем номер станции для отражения на LCD
                if (valll < -(ENCODER_INCREASE)) nowPlaying--;
                                                            // Ограничиваем изменение нулем и максимальным количеством станций
                if(nowPlaying > numberOfStations) nowPlaying = numberOfStations;
                if(nowPlaying < 0) nowPlaying = 0;

                FPrintStationNumberAndName(nowPlaying);     // Выводим на LCD предполагаемый к включению номер станции и ее название
                TimeToReturn += 2000;                       // Покрутили ручку - добавим времени а размышление
            }

            if (!(PINF & 0b00010000)){                      // Читаем пятый бит порта F - то есть ногу А5 и если она нажата (бит сброшен в 0)
//                Console.print("PINF get ");
//                Console.print(nowPlaying);
                FStart_radio(nowPlaying);                   // Вызываем новую станцию
                stepsOfProgramm = print_info;               // Меняем шаг программы
                Timer1.detachInterrupt();                   // Отключаем проверку энкодера по прерыванию
                lcd.setCursor(0,1);                         // Извещаем слушателей о смене станции
                lcd.print("  Get it now... ");
                delay(5000);                                // Ждем как лохи, чтобы станция не вызывалась часто, а также,
                                                            // чтобы поменялась информация о станции - она как раз меняется раз в 5 секунд
                lcd.setCursor(0,1);
                lcd.print(clearingRow);

                ChangeMode = PINF & 0b00010011;             // Запоминаем состояние энкодера
            }
                                                            // А это выкидывает нерешительных муфлонов из режима выбора станции
            if (millis() > TimeToReturn) stepsOfProgramm = print_info;
        }


        break;

    case print_info:                                        // Шаг программы - отражение того что играет. С него и начинаем.
        if (ChangeMode != (PINF & 0b00010011)) {            // Проверим энкодер, не нажали, не крутанули ли...
                stepsOfProgramm = set_station;              // Коль обидели - другой шаг программы
                break;                                      // Бряк, бля...
        }
//        Console.println("Case print_info");
        FPrintInfo();                                       // А не нажали - грузим LCD текущей информацией
        break;                                              // Или так, бряк...
    }
}

/*************  Печать номера и названия станции из плейлиста   ***********/

void FPrintStationNumberAndName (int stationNumber) {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("  Radio No ");
    lcd.print(stationNumber);                               // Печатаем номер желаеилй станции
    String stationName = FGetInfo(2, (byte) stationNumber); // добываем информацию о ней из плейлиста через вызов функции
    lcd.setCursor(0,1);
    lcd.print(stationName);                                 // Печатаем название
}

/*************  Запуск станции по номеру   ***********/
void FStart_radio(int val) {                                // Параметр - номер стнции или ноль
    Process setStation;                                     // Запускаем процесс
    setStation.begin("/opt/radio/start.sh");                // Объясняем ему за что схватиться
    setStation.addParameter((String)val);                   // Добавляем параметр (ноль или номер станции)
    setStation.run();                                       // Передаем в Линукс. Он запустит станцию или остановлит воспроизведение если ноль
}

/*************  Сканирование энкодера по прерыванию   ***********/

inline static void EncoderScan(void) {                      // Про это написано здесь: http://goo.gl/UYDYPB

	uint8_t newValue = PINF & 0x03;
	uint8_t fullState = newValue | EncState << 2;

	switch(fullState)
	{
		case 0x2: case 0x4: case 0xB: case 0xD:
			EncData --;
		break;
		case 0x1: case 0x7: case 0x8: case 0xE:
			EncData ++;
		break;
	}
	EncState = newValue;
}

/*************  Печать названия станции и что она играет   ***********/

void FPrintInfo(void) {
    unsigned int len_str  = 0;          // Счетчик длины строки информаци
    const char *poInfo;                 // Указатель на строку

    info = FGetInfo(1,0);               // Информация о станции
    if(info.length() < 3) {             // Если радио выключено, то на обратке - ноль.
        info = "Radio Stoped";
        RadioPlaying = false;
    }
    else {
        RadioPlaying = true;
    }

    if(info != oldInfo) {                   // Будем печатать, если что-то изменилось
        oldInfo = info;                     // Иначе просто будет дрожать экран
        lcd.setCursor(0,0);
        lcd.print(clearingRow);
        int aa = (16 - info.length()) / 2;  // Подсчеты для центровки вывода
        lcd.setCursor(aa,0);
        lcd.print(info) ;
    }

    if (RadioPlaying) {
        info = FGetInfo(0,0);               // Информация о музыке
        info.trim();                        // Очистка лишнего на всякий случай
    }
    else {
        info = "==******==";
    }

	if (info.length() > 16) {
        poInfo = info.c_str();          // Установка указателя на строку
        info += "                ";     // Добавляем справа 16 пробелов, чтобы строка уезжала с экрана полностью.
        for(byte a=0; a<16; a++) {      // Забиваем символы в стринг для вывода
            to_lcd[a] = *(poInfo+a);
        }
        lcd.setCursor(0,1);
        lcd.print(to_lcd);              // Выводим стринг
        delay(2000);                    // Ждем две секунды перед скроллингом
        len_str = 16;                   // Чтобы не двигать сначала
        while (*(poInfo+len_str))  {    // Пока не конец строки ... Поехал скроллинг
            for (byte i=0 ; i < 15 ; i++) to_lcd[i] = to_lcd[i+1] ;
            to_lcd[15] = *(poInfo+len_str);
            len_str++ ;
            for (byte i=0 ; i < 16 ; i++) { // Отображаем на LCD посимвольно.
                 lcd.setCursor(i,1);
                 lcd.print(to_lcd[i]);
            }
            if (ChangeMode != (PINF & 0b00010011)) {
                if (millis() > TimeToReturn) {
                    stepsOfProgramm = set_station;
                    break;
                }
            }
            delay(350) ;                    // Задержка перед перепечаткой
        }
    }
    else {                                  // Если информация меньше 16 символов
        if (info != oldMusicInfo) {         // и если она изменилась с прошлого раза
            oldMusicInfo = info;
            lcd.setCursor(0,1);             // Чистим строку
            lcd.print(clearingRow);
            if(info.length() <4) info = "No Info"; // Если информации вообще нет или она стремная какая-то
            int bb = (16 - info.length()) / 2;  // Центрируем
            lcd.setCursor(bb,1);
            lcd.print(info);                    // Печатаем
        }
    }
}

/*************  Добыча номера воспроизводимого радиои количества станций   ***********/
int FGetNumber(byte a) {
    int result;
    Process PrGetNumber;                                      // Обявляем процесс
    if(a==0) {
        PrGetNumber.begin("cat");                             // Объявляем реквизиты процесса
        PrGetNumber.addParameter("/opt/radio/nowNumber.txt");
    }
    else {
        PrGetNumber.begin("/opt/radio/howmany.sh");            // Объявляем реквизиты процесса
    }
    PrGetNumber.run();                                          // Запускаем процес
    String info = "";                                           // Читаем ответ в стринг
    while (PrGetNumber.available()>0) {
        char c = PrGetNumber.read();
        info += (String) c;
    }
    return atoi(info.c_str());                                  // Возвращаем число, через функцию atoi, скормив ей указатель на стринг
 }
/*************  Добыча основной инормации воспроизводимого радио   ***********/
String FGetInfo (byte arg, byte station_number) {               // Функция запроса информации
//  Console.println("In Function Now.");
    String info = "";
    Process PrGetInfo;                                          // Объявляем процесс
    switch (arg) {                                              // В зависимости от переданной информации
        case 0:                                                 // Забиваем реквизиты
            PrGetInfo.begin("cat");
            PrGetInfo.addParameter("/opt/radio/nowSong.txt");   // Информация о текущей музыке
            break;
        case 1:
            PrGetInfo.begin("cat");
            PrGetInfo.addParameter("/opt/radio/nowRadio.txt");  // Информация о текущей станции
            break;
        case 2:
            PrGetInfo.begin("/opt/radio/getStationName.sh");    // Информация о станции по номеру в плейлисте
            PrGetInfo.addParameter((String) station_number);
    }
    PrGetInfo.run();
    while (PrGetInfo.available()>0) {                           // Читаем информаци из процесса
        char c = PrGetInfo.read();
//        Console.println(c);
        info += (String) c;
    }
//    Console.println(info);
    info.trim();                                                // Чистим от лишнего
    return info;
}
